/*
 * Copyright (c) 2008, Marcus von Appen
 * All rights reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice,
 *   this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _PAPI_MSAA_H_
#define _PAPI_MSAA_H_

#include <Python.h>
#include "papi_iaccproxy.h"

/**
 * PyObject type, that encapsulates an IAccProxy.
 */
typedef struct
{
    PyObject_HEAD
    IAccProxy *obj;
    PyObject *dict;
    PyObject *prop_handlers;
    PyObject *weakrefs;
} PyIAccessible;
#define IACCPROXY(x) (((PyIAccessible *) x)->obj)
#define PAPI_IACCESSIBLE_FIRSTSLOT 0
#define PAPI_IACCESSIBLE_SLOTS 2
#ifndef PAPI_IACCESSIBLE_INTERNAL
#define PyIAccessible_Check(x) \
    (PyObject_TypeCheck (x,    \
        (PyTypeObject *)PyPapiMSAA_C_API[PAPI_IACCESSIBLE_FIRSTSLOT]))
#define PyIAccessible_New \
    (*(PyObject*(*)(void)) PyPapiMSAA_C_API[PAPI_IACCESSIBLE_FIRSTSLOT + 1])
#endif /* PAPI_IACCESSIBLE_INTERNAL */

static void **PyPapiMSAA_C_API;
#define PAPI_MSAAAPI_SLOTS \
    (PAPI_IACCESSIBLE_FIRSTSLOT + PAPI_IACCESSIBLE_SLOTS)
#define PAPI_MSAACAPI_ENTRY "_PAPI_MSAA_C_API"

static int
import_msaa (void)
{
    PyObject *_module = PyImport_ImportModule ("papi.msaa");
    if (_module != NULL)
    {
        PyObject *_capi = PyObject_GetAttrString(_module, PAPI_MSAACAPI_ENTRY);
        if (!PyCObject_Check (_capi))
        {
            Py_DECREF (_module);
            return -1;
        }
        PyPapiMSAA_C_API = (void**) PyCObject_AsVoidPtr (_capi);
        Py_DECREF (_capi);
        return 0;
    }
    return -1;
}
/**
 * Python module initialization function. This will be called upon the
 * import <module> statement and sets up the papi.msaa module.
 */
PyMODINIT_FUNC initmsaa (void);

#endif /* _PAPI_MSAA_H_ */
